# RBD INSTRUMENTS 9103 SAMPLE APPLICTION
# 9103 High Speed Sampling

# Copyright (C) 2021 RBD Instruments, Inc.

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    https://www.gnu.org/licenses/



# MODULES USED
#pyserial

import serial


#SETUP SERIAL port
#standard speed

#Query user for port number
port_number = input("Enter a port number: ")
print(port_number)

#setup port
try:
    port=serial.Serial(
        'com' + port_number,
        baudrate=230400,    #high-speed baud rate
        bytesize=serial.EIGHTBITS,
        parity=serial.PARITY_NONE,
        stopbits=serial.STOPBITS_ONE,
        xonxoff=False,
        timeout=1)
except:
    print('9103 not found at ' + port_number + '\n Press Ctrl-C to quit')
    try:
        while True:
            pass
    except KeyboardInterrupt:
            pass



# BASIC COMMANDS
# These are defaults for startup (no args)
def command_range_auto():
    port.write(message('R0')) # put in autorange

def command_default_filter_32():
    port.write(message('F032'))

def command_default_input_normal():
    port.write(message('G0'))

def command_default_bias_off():
    port.write(message('B0'))



#MESSAGE PARSING HELPERS

#Convert string to bytes for message to 9103_port
#all messages must end with newline
def message(message_string):
    return bytes('&'+ message_string + '\n','utf-8')

#Single sample (standard speed) message parsing
def parse_message_for_standard_speed_sample( msg):
    if '&s' in msg:
        return msg.strip('&')
    else:
        return ''


# Multiple sample (high speed) message parsing
# In high-speed mode, we can send the 'i' sampling command, which provides
# 10 samples per message as opposed to one in standard speed mode.
# This reduces round-trip over overhead.
# But it means that the stability, range, and unit information is
# valid for every 10 samples.
# To output in the one-message/line format as we did for standard speed, we'll parse out the info
# and format the samples so each line resembles a standard sample message
def parse_message_for_high_speed_sample( msg):
    if '&s' in msg:
        msg = msg.strip('\0')
        msg = msg.strip('&')
        list = msg.split(',')
        i = 0
        stability = ''
        range  = ''
        new_msg = ''
        units = list[-1]  # gets last item

        list.pop()  # remove last item which is  units

        for value in list:
            if i==0:
                stability = value
            elif i==1:
                range = value
            else:
                new_msg = new_msg + stability + ',' + range + ',' + value + units + '\n'
            i=i+1
        return new_msg
    else:
        return ''


#INITIALIZE 9103

command_range_auto()
command_default_filter_32()
command_default_input_normal()
command_default_bias_off()




# SAMPLING LOOP
# with keyboard interrupt (ctrl c)

# We're in high-speed mode but running a relatively slow sampling rate
# to make it easier to see what's happening in the terminal -
# since high-speed messages include 10 samples per message,
# this is one sample per second being printed to the console
print('Press Ctrl-C to stop sampling')
port.write(message('i0100'))    # lower-case 'i' commmand for high-speed messaging
# note that you can also use the "I" command for 40 sample/sec sampling or slower
#even if you are in high-speed mode

#write to file
import os

desktop_path = os.path.join(os.path.join(os.environ['USERPROFILE']), 'Desktop')

#samplefile= open('C:\\Users\\joeca\Desktop\\test.txt', 'w')
samplefile= open(desktop_path + '\\9103-sample-log.txt', 'w')


try:
    while True:
        msg=port.readline().decode('utf-8').rstrip()
        print(msg)
        parsed = parse_message_for_high_speed_sample(msg)
        if parsed:
            samplefile.write( parsed)   #no newline necessary

except KeyboardInterrupt:
    pass

samplefile.close()
port.write(message("i0000"))
print("Sampling stopped")
print("Press Ctrl-C to quit")

try:
    while True:
        pass

except KeyboardInterrupt:
    pass
